@@ -1,6 +1,6 @@
 // Copyright (C)2004 Landmark Graphics Corporation
 // Copyright (C)2005, 2006 Sun Microsystems, Inc.
-// Copyright (C)2009, 2011, 2013-2016, 2018-2020 D. R. Commander
+// Copyright (C)2009, 2011, 2013-2016, 2018-2022, 2025 D. R. Commander
 //
 // This library is free software and may be redistributed and/or modified under
 // the terms of the wxWindows Library License, Version 3.1 or (at your option)
@@ -19,6 +19,9 @@
 #define GL_GLEXT_PROTOTYPES
 #define GLX_GLXEXT_PROTOTYPES
 #include <GL/glx.h>
+#include <EGL/egl.h>
+#define EGL_EGLEXT_PROTOTYPES
+#include <EGL/eglext.h>
 #ifdef FAKEOPENCL
 #include <CL/opencl.h>
 #endif
@@ -35,6 +38,7 @@ extern "C" {
 }
 #endif
 #include "faker.h"
+#include <X11/XKBlib.h>
 #ifdef USEHELGRIND
 	#include <valgrind/helgrind.h>
 #else
@@ -47,9 +51,9 @@ extern "C" {
 	ANNOTATE_BENIGN_RACE_SIZED(&__##s, sizeof(_##s##Type), ); \
 	if(!__##s) \
 	{ \
-		vglfaker::init(); \
-		vglfaker::GlobalCriticalSection::SafeLock l(globalMutex); \
-		if(!__##s) __##s = (_##s##Type)vglfaker::loadSymbol(#s, true); \
+		faker::init(); \
+		faker::GlobalCriticalSection::SafeLock l(globalMutex); \
+		if(!__##s) __##s = (_##s##Type)faker::loadSymbol(#s, true); \
 	} \
 }
 
@@ -58,17 +62,17 @@ extern "C" {
 	ANNOTATE_BENIGN_RACE_SIZED(&__##s, sizeof(_##s##Type), ); \
 	if(!__##s) \
 	{ \
-		vglfaker::init(); \
-		vglfaker::GlobalCriticalSection::SafeLock l(globalMutex); \
-		if(!__##s) __##s = (_##s##Type)vglfaker::loadSymbol(#s); \
+		faker::init(); \
+		faker::GlobalCriticalSection::SafeLock l(globalMutex); \
+		if(!__##s) __##s = (_##s##Type)faker::loadSymbol(#s); \
 	} \
-	if(!__##s) vglfaker::safeExit(1); \
+	if(!__##s) faker::safeExit(1); \
 	if(__##s == fake_s) \
 	{ \
 		vglout.print("[VGL] ERROR: VirtualGL attempted to load the real\n"); \
 		vglout.print("[VGL]   " #s " function and got the fake one instead.\n"); \
 		vglout.print("[VGL]   Something is terribly wrong.  Aborting before chaos ensues.\n"); \
-		vglfaker::safeExit(1); \
+		faker::safeExit(1); \
 	} \
 }
 
@@ -79,10 +83,8 @@ extern "C" {
 #endif
 
 
-#define DISABLE_FAKER() \
-	vglfaker::setFakerLevel(vglfaker::getFakerLevel() + 1);
-#define ENABLE_FAKER() \
-	vglfaker::setFakerLevel(vglfaker::getFakerLevel() - 1);
+#define DISABLE_FAKER()  faker::setFakerLevel(faker::getFakerLevel() + 1);
+#define ENABLE_FAKER()  faker::setFakerLevel(faker::getFakerLevel() - 1);
 
 
 #define FUNCDEF0(RetType, f, fake_f) \
@@ -315,6 +317,20 @@ extern "C" {
 		return retval; \
 	}
 
+#define VFUNCDEF10(f, at1, a1, at2, a2, at3, a3, at4, a4, at5, a5, at6, a6, \
+	at7, a7, at8, a8, at9, a9, at10, a10, fake_f) \
+	typedef void (*_##f##Type)(at1, at2, at3, at4, at5, at6, at7, at8, at9, \
+		at10); \
+	SYMDEF(f); \
+	static INLINE void _##f(at1 a1, at2 a2, at3 a3, at4 a4, at5 a5, at6 a6, \
+		at7 a7, at8 a8, at9 a9, at10 a10) \
+	{ \
+		CHECKSYM(f, fake_f); \
+		DISABLE_FAKER(); \
+		__##f(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10); \
+		ENABLE_FAKER(); \
+	}
+
 #define FUNCDEF12(RetType, f, at1, a1, at2, a2, at3, a3, at4, a4, at5, a5, \
 	at6, a6, at7, a7, at8, a8, at9, a9, at10, a10, at11, a11, at12, a12, \
 	fake_f) \
@@ -332,6 +348,23 @@ extern "C" {
 		return retval; \
 	}
 
+#define FUNCDEF13(RetType, f, at1, a1, at2, a2, at3, a3, at4, a4, at5, a5, \
+	at6, a6, at7, a7, at8, a8, at9, a9, at10, a10, at11, a11, at12, a12, \
+	at13, a13, fake_f) \
+	typedef RetType (*_##f##Type)(at1, at2, at3, at4, at5, at6, at7, at8, at9, \
+		at10, at11, at12, at13); \
+	SYMDEF(f); \
+	static INLINE RetType _##f(at1 a1, at2 a2, at3 a3, at4 a4, at5 a5, at6 a6, \
+		at7 a7, at8 a8, at9 a9, at10 a10, at11 a11, at12 a12, at13 a13) \
+	{ \
+		RetType retval; \
+		CHECKSYM(f, fake_f); \
+		DISABLE_FAKER(); \
+		retval = __##f(a1, a2, a3, a4, a5, a6, a7, a8, a9, a10, a11, a12, a13); \
+		ENABLE_FAKER(); \
+		return retval; \
+	}
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -361,6 +394,8 @@ VFUNCDEF2(glXDestroyGLXPixmap, Display *, dpy, GLXPixmap, pix,
 FUNCDEF4(int, glXGetConfig, Display *, dpy, XVisualInfo *, vis, int, attrib,
 	int *, value, glXGetConfig)
 
+FUNCDEF0(GLXContext, glXGetCurrentContext, glXGetCurrentContext)
+
 FUNCDEF0(GLXDrawable, glXGetCurrentDrawable, glXGetCurrentDrawable)
 
 FUNCDEF2(Bool, glXIsDirect, Display *, dpy, GLXContext, ctx, glXIsDirect)
@@ -508,27 +543,6 @@ VFUNCDEF3(glXReleaseTexImageEXT, Display *, dpy, GLXDrawable, drawable,
 	int, buffer, glXReleaseTexImageEXT)
 
 
-// GLX_NV_swap_group
-
-FUNCDEF3(Bool, glXJoinSwapGroupNV, Display *, dpy, GLXDrawable, drawable,
-	GLuint, group, glXJoinSwapGroupNV)
-
-FUNCDEF3(Bool, glXBindSwapBarrierNV, Display *, dpy, GLuint, group,
-	GLuint, barrier, glXBindSwapBarrierNV)
-
-FUNCDEF4(Bool, glXQuerySwapGroupNV, Display *, dpy, GLXDrawable, drawable,
-	GLuint *, group, GLuint *, barrier, glXQuerySwapGroupNV)
-
-FUNCDEF4(Bool, glXQueryMaxSwapGroupsNV, Display *, dpy, int, screen,
-	GLuint *, maxGroups, GLuint *, maxBarriers, glXQueryMaxSwapGroupsNV)
-
-FUNCDEF3(Bool, glXQueryFrameCountNV, Display *, dpy, int, screen,
-	GLuint *, count, glXQueryFrameCountNV)
-
-FUNCDEF2(Bool, glXResetFrameCountNV, Display *, dpy, int, screen,
-	glXResetFrameCountNV)
-
-
 // GLX_SGI_swap_control
 
 FUNCDEF1(int, glXSwapIntervalSGI, int, interval, glXSwapIntervalSGI)
@@ -540,20 +554,203 @@ FUNCDEF2(GLXFBConfigSGIX, glXGetFBConfigFromVisualSGIX, Display *, dpy,
 	XVisualInfo *, vis, glXGetFBConfigFromVisualSGIX)
 
 
-// GLX_SUN_get_transparent_index
+// EGL 1.0 functions
 
-#if defined(GLX_GLXEXT_VERSION) && GLX_GLXEXT_VERSION >= 20190000
-FUNCDEF4(int, glXGetTransparentIndexSUN, Display *, dpy, Window, overlay,
-	Window, underlay, unsigned long *, transparentIndex,
-	glXGetTransparentIndexSUN)
-#else
-FUNCDEF4(int, glXGetTransparentIndexSUN, Display *, dpy, Window, overlay,
-	Window, underlay, long *, transparentIndex, glXGetTransparentIndexSUN)
-#endif
+FUNCDEF5(EGLBoolean, eglChooseConfig, EGLDisplay, display,
+	const EGLint *, attrib_list, EGLConfig *, configs, EGLint, config_size,
+	EGLint *, num_config, eglChooseConfig)
+
+FUNCDEF3(EGLBoolean, eglCopyBuffers, EGLDisplay, display, EGLSurface, surface,
+	NativePixmapType, native_pixmap, eglCopyBuffers)
+
+FUNCDEF4(EGLContext, eglCreateContext, EGLDisplay, display, EGLConfig, config,
+	EGLContext, share_context, const EGLint *, attrib_list, eglCreateContext)
+
+FUNCDEF3(EGLSurface, eglCreatePbufferSurface, EGLDisplay, display,
+	EGLConfig, config, const EGLint *, attrib_list, eglCreatePbufferSurface)
+
+FUNCDEF4(EGLSurface, eglCreatePixmapSurface, EGLDisplay, display,
+	EGLConfig, config, EGLNativePixmapType, native_pixmap,
+	const EGLint *, attrib_list, eglCreatePixmapSurface)
+
+FUNCDEF4(EGLSurface, eglCreateWindowSurface, EGLDisplay, display,
+	EGLConfig, config, NativeWindowType, native_window,
+	const EGLint *, attrib_list, eglCreateWindowSurface)
+
+FUNCDEF2(EGLBoolean, eglDestroySurface, EGLDisplay, display,
+	EGLSurface, surface, eglDestroySurface)
+
+FUNCDEF4(EGLBoolean, eglGetConfigAttrib, EGLDisplay, display,
+	EGLConfig, config, EGLint, attribute, EGLint *, value, eglGetConfigAttrib)
+
+FUNCDEF4(EGLBoolean, eglGetConfigs, EGLDisplay, display, EGLConfig *, configs,
+	EGLint, config_size, EGLint *, num_config, eglGetConfigs)
+
+FUNCDEF0(EGLDisplay, eglGetCurrentDisplay, eglGetCurrentDisplay)
+
+FUNCDEF1(EGLSurface, eglGetCurrentSurface, EGLint, readdraw,
+	eglGetCurrentSurface)
+
+FUNCDEF1(EGLDisplay, eglGetDisplay, EGLNativeDisplayType, native_display,
+	eglGetDisplay)
+
+FUNCDEF0(EGLint, eglGetError, eglGetError)
+
+typedef void (*(*_eglGetProcAddressType)(const char *))(void);
+SYMDEF(eglGetProcAddress);
+static INLINE void (*_eglGetProcAddress(const char *procName))(void)
+{
+	CHECKSYM(eglGetProcAddress, NULL);
+	return __eglGetProcAddress(procName);
+}
+
+FUNCDEF3(EGLBoolean, eglInitialize, EGLDisplay, display, EGLint *, major,
+	EGLint *, minor, eglInitialize)
+
+FUNCDEF4(EGLBoolean, eglMakeCurrent, EGLDisplay, display, EGLSurface, draw,
+	EGLSurface, read, EGLContext, context, eglMakeCurrent)
+
+FUNCDEF4(EGLBoolean, eglQueryContext, EGLDisplay, display, EGLContext, context,
+	EGLint, attribute, EGLint *, value, eglQueryContext)
+
+FUNCDEF2(const char *, eglQueryString, EGLDisplay, display, EGLint, name,
+	eglQueryString)
+
+FUNCDEF4(EGLBoolean, eglQuerySurface, EGLDisplay, display, EGLSurface, surface,
+	EGLint, attribute, EGLint *, value, eglQuerySurface)
+
+FUNCDEF2(EGLBoolean, eglSwapBuffers, EGLDisplay, display, EGLSurface, surface,
+	eglSwapBuffers)
+
+FUNCDEF1(EGLBoolean, eglTerminate, EGLDisplay, display, eglTerminate)
+
+
+// EGL 1.1 functions
+
+FUNCDEF3(EGLBoolean, eglBindTexImage, EGLDisplay, display, EGLSurface, surface,
+	EGLint, buffer, eglBindTexImage)
+
+FUNCDEF3(EGLBoolean, eglReleaseTexImage, EGLDisplay, display,
+	EGLSurface, surface, EGLint, buffer, eglReleaseTexImage)
+
+FUNCDEF4(EGLBoolean, eglSurfaceAttrib, EGLDisplay, display,
+	EGLSurface, surface, EGLint, attribute, EGLint, value, eglSurfaceAttrib)
+
+FUNCDEF2(EGLBoolean, eglSwapInterval, EGLDisplay, display, EGLint, interval,
+	eglSwapInterval)
+
+
+// EGL 1.2 functions
+
+FUNCDEF5(EGLSurface, eglCreatePbufferFromClientBuffer, EGLDisplay, display,
+	EGLenum, buftype, EGLClientBuffer, buffer, EGLConfig, config,
+	const EGLint *, attrib_list, eglCreatePbufferFromClientBuffer)
+
+
+// EGL 1.5 functions
+
+FUNCDEF4(EGLint, eglClientWaitSync, EGLDisplay, display, EGLSync, sync,
+	EGLint, flags, EGLTime, timeout, eglClientWaitSync)
+
+FUNCDEF5(EGLImage, eglCreateImage, EGLDisplay, display, EGLContext, context,
+	EGLenum, target, EGLClientBuffer, buffer, const EGLAttrib *, attrib_list,
+	eglCreateImage)
+
+FUNCDEF4(EGLSurface, eglCreatePlatformPixmapSurface, EGLDisplay, display,
+	EGLConfig, config, void *, native_pixmap, const EGLAttrib *, attrib_list,
+	eglCreatePlatformPixmapSurface)
+
+FUNCDEF3(EGLSync, eglCreateSync, EGLDisplay, display, EGLenum, type,
+	const EGLAttrib *, attrib_list, eglCreateSync)
+
+FUNCDEF2(EGLBoolean, eglDestroyImage, EGLDisplay, display, EGLImage, image,
+	eglDestroyImage)
+
+FUNCDEF2(EGLBoolean, eglDestroySync, EGLDisplay, display, EGLSync, sync,
+	eglDestroySync)
+
+FUNCDEF3(EGLDisplay, eglGetPlatformDisplay, EGLenum, platform,
+	void *, native_display, const EGLAttrib *, attrib_list,
+	eglGetPlatformDisplay)
+
+FUNCDEF4(EGLBoolean, eglGetSyncAttrib, EGLDisplay, display, EGLSync, sync,
+	EGLint, attribute, EGLAttrib *, value, eglGetSyncAttrib)
+
+FUNCDEF3(EGLBoolean, eglWaitSync, EGLDisplay, display, EGLSync, sync,
+	EGLint, flags, eglWaitSync)
+
+
+// EGL_EXT_device_query
+
+FUNCDEF3(EGLBoolean, eglQueryDisplayAttribEXT, EGLDisplay, display,
+	EGLint, attribute, EGLAttrib *, value, eglQueryDisplayAttribEXT)
+
+
+// EGL_EXT_platform_base
+
+FUNCDEF4(EGLSurface, eglCreatePlatformPixmapSurfaceEXT, EGLDisplay, display,
+	EGLConfig, config, void *, native_pixmap, const EGLint *, attrib_list,
+	eglCreatePlatformPixmapSurfaceEXT)
+
+FUNCDEF3(EGLDisplay, eglGetPlatformDisplayEXT, EGLenum, platform,
+	void *, native_display, const EGLint *, attrib_list,
+	eglGetPlatformDisplayEXT)
+
+
+// EGL_KHR_cl_event2
+
+FUNCDEF3(EGLSyncKHR, eglCreateSync64KHR, EGLDisplay, display, EGLenum, type,
+	const EGLAttribKHR *, attrib_list, eglCreateSync64KHR)
+
+
+// EGL_KHR_fence_sync
+
+FUNCDEF4(EGLint, eglClientWaitSyncKHR, EGLDisplay, display, EGLSyncKHR, sync,
+	EGLint, flags, EGLTimeKHR, timeout, eglClientWaitSyncKHR)
+
+FUNCDEF3(EGLSyncKHR, eglCreateSyncKHR, EGLDisplay, display, EGLenum, type,
+	const EGLint *, attrib_list, eglCreateSyncKHR)
+
+FUNCDEF2(EGLBoolean, eglDestroySyncKHR, EGLDisplay, display, EGLSyncKHR, sync,
+	eglDestroySyncKHR)
+
+FUNCDEF4(EGLBoolean, eglGetSyncAttribKHR, EGLDisplay, display,
+	EGLSyncKHR, sync, EGLint, attribute, EGLint *, value, eglGetSyncAttribKHR)
+
+
+// EGL_KHR_image
+
+FUNCDEF5(EGLImageKHR, eglCreateImageKHR, EGLDisplay, display,
+	EGLContext, context, EGLenum, target, EGLClientBuffer, buffer,
+	const EGLint *, attrib_list, eglCreateImageKHR)
+
+FUNCDEF2(EGLBoolean, eglDestroyImageKHR, EGLDisplay, display,
+	EGLImageKHR, image, eglDestroyImageKHR)
+
+
+// EGL_KHR_reusable_sync
+
+FUNCDEF3(EGLBoolean, eglSignalSyncKHR, EGLDisplay, display, EGLSyncKHR, sync,
+	EGLenum, mode, eglSignalSyncKHR)
+
+
+// EGL_KHR_wait_sync
+
+FUNCDEF3(EGLint, eglWaitSyncKHR, EGLDisplay, display, EGLSyncKHR, sync,
+	EGLint, flags, eglWaitSyncKHR)
 
 
 // GL functions
 
+VFUNCDEF2(glBindFramebuffer, GLenum, target, GLuint, framebuffer,
+	glBindFramebuffer)
+
+VFUNCDEF2(glBindFramebufferEXT, GLenum, target, GLuint, framebuffer,
+	glBindFramebufferEXT)
+
+VFUNCDEF2(glDeleteFramebuffers, GLsizei, n, const GLuint *, framebuffers,
+	glDeleteFramebuffers)
+
 VFUNCDEF0(glFinish, glFinish)
 
 VFUNCDEF0(glFlush, glFlush)
@@ -562,23 +759,56 @@ VFUNCDEF1(glDrawBuffer, GLenum, drawbuf, glDrawBuffer)
 
 VFUNCDEF2(glDrawBuffers, GLsizei, n, const GLenum *, bufs, glDrawBuffers)
 
+VFUNCDEF2(glFramebufferDrawBufferEXT, GLuint, framebuffer, GLenum, mode,
+	glFramebufferDrawBufferEXT)
+
+VFUNCDEF3(glFramebufferDrawBuffersEXT, GLuint, framebuffer, GLsizei, n,
+	const GLenum *, bufs, glFramebufferDrawBuffersEXT)
+
+VFUNCDEF2(glFramebufferReadBufferEXT, GLuint, framebuffer, GLenum, mode,
+	glFramebufferReadBufferEXT)
+
+VFUNCDEF2(glGetBooleanv, GLenum, pname, GLboolean *, data, glGetBooleanv)
+
+VFUNCDEF2(glGetDoublev, GLenum, pname, GLdouble *, data, glGetDoublev)
+
+VFUNCDEF2(glGetFloatv, GLenum, pname, GLfloat *, data, glGetFloatv)
+
+VFUNCDEF4(glGetFramebufferAttachmentParameteriv, GLenum, target,
+	GLenum, attachment, GLenum, pname, GLint *, params,
+	glGetFramebufferAttachmentParameteriv)
+
+VFUNCDEF3(glGetFramebufferParameteriv, GLenum, target, GLenum, pname,
+	GLint *, params, glGetFramebufferParameteriv)
+
+VFUNCDEF2(glGetIntegerv, GLenum, pname, GLint *, params, glGetIntegerv)
+
+VFUNCDEF2(glGetInteger64v, GLenum, pname, GLint64 *, data, glGetInteger64v)
+
+VFUNCDEF3(glGetNamedFramebufferParameteriv, GLuint, framebuffer,
+	GLenum, pname, GLint *, param, glGetNamedFramebufferParameteriv)
+
 FUNCDEF1(const GLubyte *, glGetString, GLenum, name, glGetString)
 
 FUNCDEF2(const GLubyte *, glGetStringi, GLenum, name, GLuint, index,
 	glGetStringi)
 
-#ifdef GL_VERSION_4_5
-
 VFUNCDEF2(glNamedFramebufferDrawBuffer, GLuint, framebuffer, GLenum, buf,
 	glNamedFramebufferDrawBuffer)
 
 VFUNCDEF3(glNamedFramebufferDrawBuffers, GLuint, framebuffer, GLsizei, n,
 	const GLenum *, bufs, glNamedFramebufferDrawBuffers)
 
-#endif
+VFUNCDEF2(glNamedFramebufferReadBuffer, GLuint, framebuffer, GLenum, mode,
+	glNamedFramebufferReadBuffer)
 
 VFUNCDEF0(glPopAttrib, glPopAttrib)
 
+VFUNCDEF1(glReadBuffer, GLenum, mode, glReadBuffer)
+
+VFUNCDEF7(glReadPixels, GLint, x, GLint, y, GLsizei, width, GLsizei, height,
+	GLenum, format, GLenum, type, GLvoid *, pixels, glReadPixels)
+
 VFUNCDEF4(glViewport, GLint, x, GLint, y, GLsizei, width, GLsizei, height,
 	glViewport)
 
@@ -658,6 +888,16 @@ FUNCDEF2(int, XNextEvent, Display *, dpy, XEvent *, xe, XNextEvent)
 
 FUNCDEF1(Display *, XOpenDisplay, _Xconst char *, name, XOpenDisplay)
 
+#ifdef LIBX11_18
+FUNCDEF6(Display *, XkbOpenDisplay, _Xconst char *, display_name,
+	int *, event_rtrn, int *, error_rtrn, int *, major_in_out,
+	int *, minor_in_out, int *, reason_rtrn, XkbOpenDisplay)
+#else
+FUNCDEF6(Display *, XkbOpenDisplay, char *, display_name, int *, event_rtrn,
+	int *, error_rtrn, int *, major_in_out, int *, minor_in_out,
+	int *, reason_rtrn, XkbOpenDisplay)
+#endif
+
 FUNCDEF5(Bool, XQueryExtension, Display *, dpy, _Xconst char *, name,
 	int *, major_opcode, int *, first_event, int *, first_error,
 	XQueryExtension)
@@ -682,6 +922,46 @@ SYMDEF(dlopen);
 
 // XCB functions
 
+FUNCDEF13(xcb_void_cookie_t, xcb_create_window, xcb_connection_t *, conn,
+	uint8_t, depth, xcb_window_t, wid, xcb_window_t, parent, int16_t, x,
+	int16_t, y, uint16_t, width, uint16_t, height, uint16_t, border_width,
+	uint16_t, _class, xcb_visualid_t, visual, uint32_t, value_mask,
+	const void *, value_list, xcb_create_window)
+
+FUNCDEF13(xcb_void_cookie_t, xcb_create_window_aux, xcb_connection_t *, conn,
+	uint8_t, depth, xcb_window_t, wid, xcb_window_t, parent, int16_t, x,
+	int16_t, y, uint16_t, width, uint16_t, height, uint16_t, border_width,
+	uint16_t, _class, xcb_visualid_t, visual, uint32_t, value_mask,
+	const xcb_create_window_value_list_t *, value_list, xcb_create_window_aux)
+
+FUNCDEF13(xcb_void_cookie_t, xcb_create_window_aux_checked,
+	xcb_connection_t *, conn, uint8_t, depth, xcb_window_t, wid,
+	xcb_window_t, parent, int16_t, x, int16_t, y, uint16_t, width,
+	uint16_t, height, uint16_t, border_width, uint16_t, _class,
+	xcb_visualid_t, visual, uint32_t, value_mask,
+	const xcb_create_window_value_list_t *, value_list,
+	xcb_create_window_aux_checked)
+
+FUNCDEF13(xcb_void_cookie_t, xcb_create_window_checked,
+	xcb_connection_t *, conn, uint8_t, depth, xcb_window_t, wid,
+	xcb_window_t, parent, int16_t, x, int16_t, y, uint16_t, width,
+	uint16_t, height, uint16_t, border_width, uint16_t, _class,
+	xcb_visualid_t, visual, uint32_t, value_mask, const void *, value_list,
+	xcb_create_window_checked)
+
+FUNCDEF2(xcb_void_cookie_t, xcb_destroy_subwindows, xcb_connection_t *, conn,
+	xcb_window_t, window, xcb_destroy_subwindows)
+
+FUNCDEF2(xcb_void_cookie_t, xcb_destroy_subwindows_checked,
+	xcb_connection_t *, conn, xcb_window_t, window,
+	xcb_destroy_subwindows_checked)
+
+FUNCDEF2(xcb_void_cookie_t, xcb_destroy_window, xcb_connection_t *, conn,
+	xcb_window_t, window, xcb_destroy_window)
+
+FUNCDEF2(xcb_void_cookie_t, xcb_destroy_window_checked,
+	xcb_connection_t *, conn, xcb_window_t, window, xcb_destroy_window_checked)
+
 FUNCDEF2(const xcb_query_extension_reply_t *, xcb_get_extension_data,
 	xcb_connection_t *, conn, xcb_extension_t *, ext, xcb_get_extension_data)
 
@@ -712,13 +992,21 @@ FUNCDEF1(xcb_generic_event_t *, xcb_wait_for_event, xcb_connection_t *, conn,
 
 VFUNCDEF2(glBindBuffer, GLenum, target, GLuint, buffer, NULL)
 
+VFUNCDEF2(glBindRenderbuffer, GLenum, target, GLuint, renderbuffer, NULL)
+
 VFUNCDEF7(glBitmap, GLsizei, width, GLsizei, height, GLfloat, xorig,
 	GLfloat, yorig, GLfloat, xmove, GLfloat, ymove, const GLubyte *, bitmap,
 	NULL)
 
+VFUNCDEF10(glBlitFramebuffer, GLint, srcX0, GLint, srcY0, GLint, srcX1,
+	GLint, srcY1, GLint, dstX0, GLint, dstY0, GLint, dstX1, GLint, dstY1,
+	GLbitfield, mask, GLenum, filter, NULL)
+
 VFUNCDEF4(glBufferData, GLenum, target, GLsizeiptr, size, const GLvoid *, data,
 	GLenum, usage, NULL)
 
+FUNCDEF1(GLenum, glCheckFramebufferStatus, GLenum, target, NULL)
+
 VFUNCDEF1(glClear, GLbitfield, mask, NULL)
 
 VFUNCDEF4(glClearColor, GLclampf, red, GLclampf, green, GLclampf, blue,
@@ -727,19 +1015,25 @@ VFUNCDEF4(glClearColor, GLclampf, red, GLclampf, green, GLclampf, blue,
 VFUNCDEF5(glCopyPixels, GLint, x, GLint, y, GLsizei, width, GLsizei, height,
 	GLenum, type, NULL)
 
+VFUNCDEF2(glDeleteRenderbuffers, GLsizei, n, const GLuint *, renderbuffers,
+	NULL)
+
 VFUNCDEF0(glEndList, NULL)
 
+VFUNCDEF4(glFramebufferRenderbuffer, GLenum, target, GLenum, attachment,
+	GLenum, renderbuffertarget, GLuint, renderbuffer, NULL)
+
 VFUNCDEF2(glGenBuffers, GLsizei, n, GLuint *, buffers, NULL)
 
+VFUNCDEF2(glGenFramebuffers, GLsizei, n, GLuint *, ids, NULL)
+
+VFUNCDEF2(glGenRenderbuffers, GLsizei, n, GLuint *, renderbuffers, NULL)
+
 VFUNCDEF3(glGetBufferParameteriv, GLenum, target, GLenum, value, GLint *, data,
 	NULL)
 
 FUNCDEF0(GLenum, glGetError, NULL)
 
-VFUNCDEF2(glGetFloatv, GLenum, pname, GLfloat *, params, NULL)
-
-VFUNCDEF2(glGetIntegerv, GLenum, pname, GLint *, params, NULL)
-
 VFUNCDEF0(glLoadIdentity, NULL)
 
 FUNCDEF2(void *, glMapBuffer, GLenum, target, GLenum, access, NULL)
@@ -759,14 +1053,32 @@ VFUNCDEF0(glPushMatrix, NULL)
 
 VFUNCDEF2(glRasterPos2i, GLint, x, GLint, y, NULL)
 
-VFUNCDEF1(glReadBuffer, GLenum, mode, NULL)
+VFUNCDEF4(glRenderbufferStorage, GLenum, target, GLenum, internalformat,
+	GLsizei, width, GLsizei, height, NULL)
 
-VFUNCDEF7(glReadPixels, GLint, x, GLint, y, GLsizei, width, GLsizei, height,
-	GLenum, format, GLenum, type, GLvoid *, pixels, NULL)
+VFUNCDEF5(glRenderbufferStorageMultisample, GLenum, target, GLsizei, samples,
+	GLenum, internalformat, GLsizei, width, GLsizei, height, NULL)
 
 FUNCDEF1(GLboolean, glUnmapBuffer, GLenum, target, NULL)
 
-FUNCDEF0(GLXContext, glXGetCurrentContext, NULL)
+// EGL functions used by the faker (but not interposed.)
+
+FUNCDEF1(EGLBoolean, eglBindAPI, EGLenum, api, NULL)
+
+FUNCDEF2(EGLBoolean, eglDestroyContext, EGLDisplay, display,
+	EGLContext, context, NULL)
+
+FUNCDEF0(EGLContext, eglGetCurrentContext, NULL)
+
+FUNCDEF3(EGLBoolean, eglQueryDevicesEXT, EGLint, max_devices, EGLDeviceEXT *,
+	devices, EGLint *, num_devices, NULL)
+
+FUNCDEF2(const char *, eglQueryDeviceStringEXT, EGLDeviceEXT, device, EGLint,
+	name, NULL)
+
+FUNCDEF0(EGLenum, eglQueryAPI, NULL)
+
+FUNCDEF0(EGLBoolean, eglReleaseThread, NULL)
 
 // We load all XCB functions dynamically, so that the same VirtualGL binary
 // can be used to support systems with and without XCB libraries.
@@ -807,19 +1119,4 @@ FUNCDEF3(xcb_keysym_t, xcb_key_symbols_get_keysym, xcb_key_symbols_t *, syms,
 #endif
 
 
-static INLINE int DrawingToFront(void)
-{
-	GLint drawbuf = GL_BACK;
-	_glGetIntegerv(GL_DRAW_BUFFER, &drawbuf);
-	return IS_FRONT(drawbuf);
-}
-
-
-static INLINE int DrawingToRight(void)
-{
-	GLint drawbuf = GL_LEFT;
-	_glGetIntegerv(GL_DRAW_BUFFER, &drawbuf);
-	return IS_RIGHT(drawbuf);
-}
-
 #endif  // __FAKER_SYM_H__
